/*
 * $Id$
 * 
 *          This source code is derived from, but NOT part of GROMACS
 * 		pls direct comments to tsirk@vt.edu
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 */

/* This line is only for CVS version info */
static char *SRCID_template_c = "$Id$";

extern "C"
{
#include "statutil.h"
#include "typedefs.h"
#include "smalloc.h"
#include "vec.h"
#include "copyrite.h"
#include "statutil.h"
#include "tpxio.h"
}

float* callcuda(int, real*);
void print2File(float* myresults);


int main(int argc,char *argv[])
{
  static char *desc[] = {
    "This is a template program for GPU acclerated functions. Use it to create ",
    "your own GPU-accelerated GROMACS tools. It reads coordinates into a CUDA kernal,",
    " and copies them back as results."
  };
  
  static int n=1;

  /* Extra arguments - but note how you always get the begin/end
   * options when running the program, without mentioning them here!
   */
 
  t_pargs pa[] = {
    { "-n", FALSE, etINT, {&n},
      "Plot data for atom number n (starting on 1)"
    }
  };
  
  t_topology top;
  int        ePBC;
  char       title[STRLEN];
  t_trxframe fr;
  rvec       *xtop;
  matrix     box;
  int        status;
  int        flags = TRX_READ_X;

  t_filenm fnm[] = {
    { efTPS,  NULL,  NULL, ffREAD },   /* this is for the topology */
    { efTRX, "-f", NULL, ffREAD }      /* and this for the trajectory */
  };
  
#define NFILE asize(fnm)

//  CopyRight(stderr,argv[0]);

	// This is the routine responsible for adding default options,
	// calling the X/motif interface, etc.

  parse_common_args(&argc,argv,PCA_CAN_TIME | PCA_CAN_VIEW,
		    NFILE,fnm,asize(pa),pa,asize(desc),desc,0,NULL);

	// You can access coordinates, names, charges, etc
	// printf("Coordinates at t=%8.3f : %8.5f %8.5f %8.5f\n",fr.time,fr.x[n][XX],fr.x[n][YY],fr.x[n][ZZ]);  
	// printf("Atom name: %s\n",*(top.atoms.atomname[n]));
	// printf("Atom charge: %f\n",top.atoms.atom[n].q);
  
  read_tps_conf(ftp2fn(efTPS,NFILE,fnm),title,&top,&ePBC,&xtop,NULL,box,TRUE);
  sfree(xtop);
  
//*******************write cooridnates example*****************//


  // The first time we read data is a little special
  read_first_frame(&status,ftp2fn(efTRX,NFILE,fnm),&fr,flags);

  // This is the main loop over frames

	float* results; //your results

  do {

	// call function that executes cuda kernel
	// pass pointer to coordinates
	// could also pass fr.v, fr.f, etc

	results = callcuda(top.atoms.nr, (float*)fr.x); 

	//print 'em out
	print2File(results);

	// free pointers
	free(results);

  } while(read_next_frame(status,&fr));


  thanx(stderr);
  
  return 0;
}

void print2File(float* myresults)
{
	printf("\nx0: %f y0: %f z0: %f\n", myresults[0], myresults[1], myresults[2]);
}

